import {
  world,
  Player,
  BeforeChatEvent,
  Location,
  ItemStack,
  system,
  Items
} from "@minecraft/server"
import {
  ActionFormData,
  MessageFormData,
  ModalFormData,
  ActionFormResponse,
  MessageFormResponse,
  ModalFormResponse
} from "@minecraft/server-ui";
import { Database } from "./Modules/Database.js"
import { CommandBuild } from "./Modules/CommandBuilder.js";
import { compareString, getItemname, random } from "./Modules/Utility.js"
import { prefix, currency, itemShop, starterMoney, maxMoney } from "./Configuration.js";

let MoneyDB = new Database("moneyDB")
let HomeDB = new Database("homeDB")
let WarpDB = new Database("warpDB")
let BackDB = new Database("backDB")
let BanDB = new Database("banDB")
let ShopDB = new Database("shopDB")
let ServerSetDB = new Database("settingDB")

const playerOnline = []
const tpaCooldown = []
const backCooldown = []

world.events.worldInitialize.subscribe(d => {
  if (getSetting("itemShop") == undefined) {
    ServerSetDB.set("itemShop", itemShop)
  }
})

system.events.beforeWatchdogTerminate.subscribe(data => {
  data.cancel = true
})

world.events.playerJoin.subscribe(async data => {
  if (BanDB.get(data.player.nameTag) != undefined) return runCommand(`kick "${data.player.nameTag}" \n§4§lYou're have been banned\nReason : §e${BanDB.get(data.player.nameTag).reason} \n§4By : §e${BanDB.get(data.player.nameTag).by}`)
  await getMoney(data.player)
  if (getMoney(data.player) == NaN) {
    updateMoney(data.player, getSetting("starterMoney") || starterMoney)
  }
  if (data.player.getTags().find(t => t.startsWith("tpasetting")) == undefined) {
    data.player.addTag("tpasetting:on")
  }
})

world.events.playerLeave.subscribe(async data => {
  playerOnline.splice(playerOnline.findIndex(p => p.name == data.playerName), 1)
})

world.events.tick.subscribe(async data => {
  let tick = data.currentTick
  if (tick % 1200 == 0) {
    let date = new Date()
    let hour = date.getHours()
    let laststock = ShopDB.get("lastStock") || 24
    if (hour != laststock) {
      refreshStock()
    }
  }
  if (tick % 20 == 0) {
    for (const p of tpaCooldown) {
      if (p.cooldown - 1 >= 1) {
        p.cooldown = p.cooldown - 1
      } else {
        tpaCooldown.splice(tpaCooldown.findIndex(l => l == p), 1)
      }
    }
    for (const p of backCooldown) {
      if (p.cooldown - 1 >= 1) {
        p.cooldown = p.cooldown - 1
      } else {
        backCooldown.splice(backCooldown.findIndex(l => l == p), 1)
      }
    }
    for (const player of world.getPlayers()) {
      if (playerOnline.find(p => p.name == player.nameTag) == undefined) {
        playerOnline.push({
          name: player.nameTag,
          online: 0
        })
      } else {
        playerOnline[playerOnline.findIndex(p => p.name == player.nameTag)].online += 1
      }
    }
  }
  for (const player of world.getPlayers()) {
    let banTag = player.getTags().find(t => t.toLowerCase().startsWith("ban-"))
    if (banTag != undefined) {
      if (BanDB.has(player.nameTag)) continue;
      let reason = banTag.split("-")[1]
      await BanDB.set(player.nameTag, {
        reason: reason,
        by: "System"
      })
      player.removeTag(banTag)
      runCommand(`kick "${player.nameTag}" \n§4§lYou're have been banned\nReason : §e${reason} \n§4By : §eSystem`)
    }
    if (player.getComponent("health").current >= 0 && player.hasTag("died")) {
      player.removeTag("died")
      player.tell(`§eYou just died! use §a${prefix}back§e command for teleport to died location.`)
    }
  }
})

world.events.entityHurt.subscribe(async data => {
  let entityHurt = data.hurtEntity
  if (entityHurt.typeId === "minecraft:player" && entityHurt.getComponent("health").current <= 0) {
    entityHurt.addTag("died")
    const backData = {
      x: Math.floor(entityHurt.location.x),
      y: Math.floor(entityHurt.location.y),
      z: Math.floor(entityHurt.location.z),
      dimension: getCurrentDimension(entityHurt)
    }
    BackDB.set(`Back-${entityHurt.nameTag}`, backData)
    if (backCooldown.find(b => b.name == entityHurt.nameTag) != undefined) {
      backCooldown.splice(backCooldown.findIndex(b => b.name == entityHurt.nameTag), 1)
    }
  }
  let lowEntity = [
    "minecraft:pig",
    "minecraft:cow",
    "minecraft:sheep",
    "minecraft:chicken",
    "minecraft:cod",
    "minecraft:salmon",
    "minecraft:pufferfish",
    "minecraft:tropicalfish"
  ]
  let mediumEntity = [
    "minecraft:zombie",
    "minecraft:husk",
    "minecraft:skeleton",
    "minecraft:stray",
    "minecraft:blaze",
    "minecraft:zombie_villager",
    "minecraft:zombie_villager_v2",
    "minecraft:pillager",
    "minecraft:vex",
    "minecraft:evocation_illager",
    "minecraft:slime",
    "minecraft:drowned",
    "minecraft:guardian",
    "minecraft:iron_golem",
    "minecraft:spider",
    "minecraft:magma_cube",
    "minecraft:cave_spider",
    "minecraft:endermite",
    "minecraft:piglin",
    "minecraft:piglin_brute",
    "minecraft:zoglin",
    "minecraft:shulker",
    "minecraft:enderman",
    "minecraft:phantom",
    "minecraft:creeper",
    "minecraft:wither_skeleton",
    "minecraft:ghast",
    "minecraft:witch",
    "minecraft:zombie_pigman"
  ]
  let highEntity = [
    "minecraft:warden",
    "minecraft:ravager",
    "minecraft:wither",
    "minecraft:ender_dragon"
  ]
  if (data.cause != "none" && data.damagingEntity.typeId === "minecraft:player" && entityHurt.getComponent("health").current <= 0) {
    let player = data.damagingEntity
    let get = 0
    if (lowEntity.includes(entityHurt.typeId)) {
      if (getSetting("earnMoneyfromMobs") == "false" || getSetting("moneySystem") == "false") return;
      get = random(1, 5)
    }
    if (mediumEntity.includes(entityHurt.typeId)) {
      if (getSetting("earnMoneyfromMobs") == "false" || getSetting("moneySystem") == "false") return;
      get = random(20, 50)
    }
    if (highEntity.includes(entityHurt.typeId)) {
      if (getSetting("earnMoneyfromMobs") == "false" || getSetting("moneySystem") == "false") return;
      get = random(70, 100)
    }
    if (entityHurt.typeId === "minecraft:player") {
      if (getSetting("earnMoneyfromPlayers") == "false" || getSetting("moneySystem") == "false") return;
      let targetMoney = await getMoney(entityHurt)
      if (targetMoney <= 0) {
        get = 0
      } else {
        get = Math.floor(targetMoney / 3)
      }
      await updateMoney(entityHurt, targetMoney - get)
      entityHurt.tell(`§aYou just killed and lost §e${currency}${get.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    }
    if (get <= 0) return;
    let playerMoney = await getMoney(player)
    let max = getSetting("maxMoney") || maxMoney
    if (playerMoney >= max) return;
    if (playerMoney + get > max) get = max - playerMoney
    await updateMoney(player, playerMoney + get)
    player.onScreenDisplay.setActionBar(`§aYou got §e${currency}${get.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
  }
})

world.events.beforeChat.subscribe(data => {
  if (data.message.startsWith(prefix)) {
    let args = data.message.slice(prefix.length).trim().split(' ');
    let command = args.shift().toLowerCase();
    const player = data.sender;
    data.cancel = true
    let getCommand = CommandBuild.getAllRegistation().find(c => c.name == command || (c.aliases && c.aliases.includes(command)))
    if (getCommand != undefined) {
      if (getCommand.admin && !player.hasTag("admin")) return player.tell("§4You're not admin!")
      if (getSetting(`${getCommand.group.toLowerCase()}System`) === "false")
        return player.tell(`§4${getCommand.group} System is disabled!`)
      if (getSetting(`${getCommand.name.toLowerCase()}System`) === "false")
        return player.tell(`§4${getCommand.name.charAt(0).toUpperCase() + getCommand.name.slice(1)} System is disabled!`)
      getCommand.callback(data, player, args)
    } else {
      player.tell(`§4There's no command with that name! Use ${prefix}help to get command list.`)
    }
  }
})

// -- FUNCTION
async function runCommand(command, dim) {
  let dimension = dim || "overworld"
  try {
    return { error: false, ...world.getDimension(dimension).runCommandAsync(command) };
  } catch (error) {
    return { error: true };
  }
}

/**
 * @param {String} playerName
 * @returns {Player | undefined}
 */
function getPlayer(playerName) {
  let allPlayers = world.getPlayers()
  let name = playerName.replaceAll("_", " ").toLowerCase()
  let targetPlayer = undefined;
  for (const p of allPlayers) {
    if (name == p.name.toLowerCase() || name == p.nameTag.toLowerCase()) {
      targetPlayer = p
    }
  }
  return targetPlayer
}

const SettingType = {
  TRUEFALSE: 0,
  CUSTOMSTR: 1,
  CUSTOMNUM: 2
}
function getSetting(name) {
  let value = ServerSetDB.get(name)
  if (value == undefined) return undefined
  return value
}

async function settingShop(value) {
  world.scoreboard.getObjective("DB_settingDB").getParticipants().forEach(async e => {
    await runCommand(`scoreboard players reset "${e.displayName}" "DB_settingDB"`)
  })
  ServerSetDB.set("itemShop", value)
}

async function tpPlayer(player, destination, dim) {
  let dimension = dim || getCurrentDimension(player)
  let pos = destination
  if (destination.nameTag != undefined) {
    pos = `"${destination.nameTag}"`
  }
  await setGamerule("sendCommandFeedback", false)
  await runCommand(`tp "${player.nameTag}" ${pos}`, dimension)
  await setGamerule("sendCommandFeedback", true)
}

/**
 * 
 * @param {Player} player 
 * @param {ActionFormData | MessageFormData | ModalFormData} form 
 * @returns {ActionFormResponse | MessageFormResponse | ModalFormResponse}
 */
async function ForceOpen(player, form) {
  while (true) {
    const response = await form.show(player);
    if (response.cancelationReason !== "userBusy") return response;
  }
}

function getCurrentDimension(player) {
  let dimension = player.dimension.id.replace("minecraft:", "")
  return dimension;
}

function setGamerule(name, value) {
  runCommand(`gamerule ${name} ${value}`)
}

async function getItemCanTake(inventory, itemid, maxStack) {
  maxStack = maxStack || 64
  if (inventory.emptySlotsCount >= 1) {
    return maxStack
  } else {
    let canTake = 0
    for (let i = 0; i < inventory.size; i++) {
      let item = inventory.getItem(i)
      if (item.typeId == itemid) {
        canTake = canTake + (maxStack - item.amount)
      }
    }
    return canTake
  }
}

// -- [GENERAL] -- \\

// HELP
CommandBuild.register({
  name: "help",
  description: "Help List / Command List",
  usage: "help <command_name?>"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  (data, player, args) => {
    if (!args[0]) {
      let allCommand = CommandBuild.getAllRegistation()
      let commandGroup = []
      for (const cmd of allCommand) {
        if (!commandGroup.includes(cmd.group)) {
          commandGroup.push(cmd.group)
        }
      }
      let helpMessage = `§eHelp Message ("?" That means it's optional or not must be filled)`
      for (const group of commandGroup) {
        if (group == "Admin" && !player.hasTag("admin")) continue;
        if (getSetting(`${group.toLowerCase()}System`) && getSetting(`${group.toLowerCase()}System`) === "false") continue;
        let cmd = allCommand.filter(c => c.group == group)
        helpMessage += `\n §a${group} Command :`
        for (const command of cmd) {
          if (getSetting(`${command.name.toLowerCase()}System`) && getSetting(`${command.name.toLowerCase()}System`) === "false") continue;
          helpMessage += `\n §e - §a${prefix}${command.name} §e| ${prefix}${command.usage} (${command.description})`
        }
      }
      player.tell(helpMessage)
    } else {
      let command = CommandBuild.getAllRegistation().find(c => c.name == args[0] || (c.aliases && c.aliases.includes(args[0])))
      if (command == undefined) return player.tell("§4There's no command with that name")
      if (command.group == "Admin" && !player.hasTag("admin")) return player.tell("§4You're not admin!")
      if (getSetting(`${command.name.toLowerCase()}System`) && getSetting(`${command.name.toLowerCase()}System`) === "false")
        return player.tell(`§4${command.name.charAt(0).toUpperCase() + command.name.slice(1)} System is disabled!`)
      let helpMessage = `§a${command.name[0].toUpperCase() + command.name.substring(1)} Command :
§a Usage : §e${prefix}${command.usage || command.name}
§a Description : §e${command.description}`
      if (command.aliases) {
        helpMessage += `\n§a Aliases: §e${JSON.stringify(command.aliases)}`
      }
      player.tell(helpMessage)
    }
  }
)

// BACK
CommandBuild.register({
  name: "back",
  description: "Teleport to last died location",
  usage: "back"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let backData = BackDB.get(`Back-${player.nameTag}`)
    if (backData != undefined) {
      if (backCooldown.find(p => p.name == player.nameTag) != undefined) return player.tell(`§4You just use tpa command! In cooldown for §e${backCooldown.find(p => p.name == player.nameTag).cooldown}s`)
      player.tell("§eTeleporting...")
      await tpPlayer(player, `${backData.x} ${backData.y} ${backData.z}`, backData.dimension)
      player.tell("§aSuccessfully Teleport.")
      backCooldown.push({
        name: player.nameTag,
        cooldown: 300
      })
    } else {
      player.tell("§4You don't have died location!")
    }
  }
)

// PLAYERLIST
CommandBuild.register({
  name: "playerlist",
  description: "Show player list",
  usage: "playerlist"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let message = "§a[PLAYER LIST] : "
    for (const plr of playerOnline) {
      message += `\n§e - ${plr.name} |§a Online for`
      if (plr.online >= 86400) {
        let day = Math.floor(plr.online / 86400)
        message += ` §e${day} §adays,`
      }
      if (plr.online >= 3600) {
        let hour = Math.floor(plr.online / 3600)
        message += ` §e${hour % 24} §ahours,`
      }
      if (plr.online >= 60) {
        let minute = Math.floor(plr.online / 60)
        message += ` §e${minute % 60} §aminutes,`
      }
      let second = plr.online
      message += ` §e${second % 60} §aseconds`
    }
    player.tell(message)
  }
)

// -- [MONEY] -- \\
/**
 * @param {Player} player 
 * @returns {Number}
 */
async function getMoney(player) {
  let playerMoney = await MoneyDB.get(`Balance-${player.nameTag}`)
  if (playerMoney == undefined) {
    await MoneyDB.set(`Balance-${player.nameTag}`, getSetting("starterMoney") || starterMoney)
    return getSetting("starterMoney") || starterMoney
  } else {
    return playerMoney
  }
}

/**
 * @param {Player} player 
 * @param {String | Number} value 
 */
async function updateMoney(player, value) {
  await MoneyDB.set(`Balance-${player.nameTag}`, value)
}

async function refreshStock() {
  let shopSetting = getSetting("refreshStock") || "true"
  let shop = {}
  const itemShop = getSetting("itemShop") || itemShop
  for (const category of Object.keys(itemShop)) {
    if (!shop[category]) {
      shop[category] = []
    }
    let amount
    if (itemShop[category].length <= 1) {
      amount = itemShop[category].length
    } else {
      amount = random(Math.floor(itemShop[category].length / 2), itemShop[category].length)
    }
    if (shopSetting == "false")
     amount = itemShop[category].length
    for (let i = 0; i < amount;) {
      let item = itemShop[category][random(0, itemShop[category].length - 1)]
      if (shop[category].find(s => s == item.id) != undefined) continue;
      shop[category].push(item.id)
      i++
    }
  }
  let date = new Date()
  let hour = date.getHours()
  ShopDB.set("currentStock", shop)
  ShopDB.set("lastStock", hour)
}

// MONEY
CommandBuild.register({
  name: "money",
  description: "See your's money or someone's money",
  aliases: ["balance"],
  usage: "money <player_name?>",
  group: "Money"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) {
      let playerMoney = await getMoney(player)
      player.tell(`§aYour's money : §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    } else {
      let targetPlayer = await getPlayer(args[0])
      if (targetPlayer != undefined) {
        let targetMoney = await getMoney(targetPlayer)
        player.tell(`§a${targetPlayer.nameTag}'s money : §e${currency}${await targetMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
      } else {
        let playerMoney = await getMoney(player)
        player.tell(`§aYour's money : §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
      }
    }
  }
)

// PAY
CommandBuild.register({
  name: "pay",
  description: "Pay or Transfer money to other player",
  aliases: ["transfer"],
  usage: "pay <player_name>",
  group: "Money"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    if (!args[1]) return player.tell("§4You must input the amount!")
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      let max = getSetting("maxMoney") || maxMoney
      if (targetPlayer == player) return player.tell("§4You can't pay to yourself!")
      let amount = Number(args[1])
      if (!Number.isInteger(amount)) return player.tell("§4You must input number!")
      if (amount <= 0) return player.tell("§4You must input the amount more than 1!")
      let playerMoney = await getMoney(player)
      let targetMoney = await getMoney(targetPlayer)
      if (amount > playerMoney) return player.tell("§4You don't have enough money!")
      if (targetMoney >= max) return player.tell(`§4${targetPlayer.nameTag}'s money is max!`)
      if (targetMoney + amount > max) amount = max - targetMoney
      playerMoney = playerMoney - amount
      targetMoney = targetMoney + amount
      await updateMoney(player, playerMoney)
      await updateMoney(targetPlayer, targetMoney)
      player.tell(`§aSuccessfully paid to ${targetPlayer.nameTag} for §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}.`)
      targetPlayer.tell(`§a${player.nameTag} just paid to you for §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}.`)
    } else {
      player.tell("§4There's no player with that name!")
    }
  }
)

// TOP MONEY
CommandBuild.register({
  name: "topmoney",
  description: "See top money leaderboard",
  usage: "topmoney",
  group: "Money"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let BalanceData = []
    MoneyDB.forEach((key, value) => {
      let playerName = key.substring("Balance-".length)
      if (playerName != undefined) {
        let playerMoney = value
        let moneyData = {
          playerName: playerName,
          playerMoney: playerMoney
        }
        BalanceData.push(moneyData)
      }
    })
    if (BalanceData.length <= 0) return player.tell("§4There's no player with money!")
    BalanceData.sort((a, b) => b.playerMoney - a.playerMoney)
    let message = "§a[TOP MONEY LEADERBOARD] :"
    for (let i = 0; i < BalanceData.length; i++) {
      let playerName = BalanceData[i].playerName
      let playerMoney = Number(BalanceData[i].playerMoney)
      message += `\n§a${i + 1}. ${playerName}: §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`
    }
    player.tell(message)
  }
)

// SHOP
CommandBuild.register({
  name: "shop",
  description: "Buy something",
  usage: "shop",
  group: "Money"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    shop(player)
  }
)
/**
 * @param {Player} player 
 */
async function shop(player) {
  if (ShopDB.get("currentStock") == undefined) {
    await refreshStock()
  }
  let itemStock = ShopDB.get("currentStock")
  let itemGroup = Object.keys(itemStock)
  let mainForm = new ActionFormData()
    .title("§l§eSHOP")
    .body("Select item type")
  itemGroup.forEach(k => mainForm.button(`§l§e${k} §e(§a${itemStock[k].length}§e)`))
  player.tell("§eClose chat for Shop UI Open!")

  let res = await ForceOpen(player, mainForm)
  if (!res.canceled) {
    let itemShop = getSetting("itemShop") || itemShop
    let selectedGroup = itemGroup[res.selection]
    let items = []
    let shopForm = new ActionFormData()
      .title(`§l§e${selectedGroup.toUpperCase()} SHOP`)
      .body(`Select Items`)
    itemStock[selectedGroup].forEach(item => {
      item = itemShop[selectedGroup].find(i => i.id == item)
      if (item != undefined) items.push(item)
    })
    items.sort((a, b) => compareString(a, b, "displayname"))
    items.forEach(item => {
      shopForm.button(`§l§aName : ${item.displayname}\n§l§ePrice : ${currency}${item.price.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    })
    shopForm.button("§l§4<== BACK")
    res = await ForceOpen(player, shopForm)
    if (!res.canceled) {
      let item = items[res.selection]
      if (item == undefined) return shop(player)
      shop_ItemUI(player, item)
    }
  }
}
/**
 * @param {Player} player 
 * @param {Array} item 
 */
async function shop_ItemUI(player, item) {
  let canTake = player.getComponent('inventory').container.emptySlotsCount > 0
  if (!canTake) return player.tell("§4You must have at least 1 empty slot!")
  let itemForm = new ModalFormData()
    .title(`${item.displayname} Shop`)
    .slider("Amount", 1, 64, 1)
  let res = await ForceOpen(player, itemForm)
  if (!res.canceled) {
    let [slider] = res.formValues;
    let amount = slider
    shop_ConfirmUI(player, item, amount)
  }
}
/**
 * @param {Player} player 
 * @param {Array} item 
 * @param {Number} amount 
 */
async function shop_ConfirmUI(player, item, amount) {
  let price = item.price * amount
  let itemName = item.displayname
  let confirmUI = new MessageFormData()
    .title("§l§ePURHASE CONFIRMATION")
    .body(`Are you sure want to buy ${itemName} for §e${price.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}§r?`)
    .button1("§l§aSURE")
    .button2("§l§4CANCEL")
  let res = await ForceOpen(player, confirmUI)
  if (!res.canceled) {
    if (res.selection == 1) {
      let playerMoney = await getMoney(player)
      if (price > playerMoney) return player.tell("§4You don't have enough money!")
      let itemStack = new ItemStack(Items.get(item.itemid), amount, item.data)
      let inventory = player.getComponent('inventory').container
      inventory.addItem(itemStack)
      playerMoney = playerMoney - price
      await updateMoney(player, playerMoney)
      player.tell(`§eSuccessfully bought §ax${amount} ${itemName}§e!`)
    }
  }
}

// -- [HOME] -- \\

// HOME
CommandBuild.register({
  name: "home",
  description: "Teleport to selected home",
  usage: "home <home_name>",
  group: "Home"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    //if(getSetting("homeSystem") && getSetting("homeSystem") == "true") return player.tell("§Home System is disabled!")
    if (!args[0]) return player.tell("§4You must input the home name!")
    let name = args[0]
    let playerHome
    HomeDB.forEach((key, value) => {
      if (key.startsWith(`Home-${player.nameTag}`) && key.substring(`Home-${player.nameTag}_`.length).toLowerCase() == name.toLowerCase()) {
        playerHome = value
        return;
      }
    })
    if (playerHome != undefined) {
      player.tell("§eTeleporting...")
      await tpPlayer(player, `${playerHome.x} ${playerHome.y} ${playerHome.z}`, playerHome.dimension)
      player.tell("§aSuccessfully Teleport.")
    } else {
      player.tell("§4You don't have home with that name!")
    }
  }
)

// SETHOME
CommandBuild.register({
  name: "sethome",
  description: "Add or Set home with selected name",
  usage: "sethome <home_name>",
  group: "Home"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the home name!")
    let name = args[0]
    let playerHome
    HomeDB.forEach((key, value) => {
      if (key.startsWith(`Home-${player.nameTag}`) && key.substring(`Home-${player.nameTag}_`.length).toLowerCase() == name.toLowerCase()) {
        playerHome = value
        return;
      }
    })
    if (playerHome != undefined) return player.tell("§4You already have home with that name!")
    const homeObject = {
      x: Math.floor(player.location.x),
      y: Math.floor(player.location.y),
      z: Math.floor(player.location.z),
      dimension: getCurrentDimension(player)
    }
    await HomeDB.set(`Home-${player.nameTag}-${name}`, homeObject)
    player.tell(`§aSuccessfully add home with name ${name}!`)
  }
)

// DELHOME
CommandBuild.register({
  name: "delhome",
  description: "Delete or Remove home",
  usage: "delhome <home_name>",
  group: "Home"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the home name!")
    let name = args[0]
    let playerHome
    HomeDB.forEach((key, value) => {
      if (key.startsWith(`Home-${player.nameTag}`) && key.substring(`Home-${player.nameTag}_`.length).toLowerCase() == name.toLowerCase()) {
        playerHome = key
        return;
      }
    })
    if (playerHome != undefined) {
      await HomeDB.delete(playerHome)
      player.tell(`§aSuccessfully remove home with name ${name}!`)
    } else {
      player.tell("§4You don't have home with that name!")
    }
  }
)

// LISTHOME
CommandBuild.register({
  name: "listhome",
  description: "Get all list of your's home",
  usage: "listhome",
  group: "Home"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let message = ""
    HomeDB.forEach((key, value) => {
      if (key.startsWith(`Home-${player.nameTag}`)) {
        let home = value
        let hname = key.substring(`Home-${player.nameTag}_`.length)
        message += `\n§e  -§a ${hname}§e ${home.x}, ${home.y}, ${home.z} | ${home.dimension}`;
      }
    })
    if (message != "") {
      player.tell(`§eHome List : ${message}`)
    }
    else {
      player.tell("§4You don't have home!")
    }
  }
)

// -- [WARP] -- \\

// WARP
CommandBuild.register({
  name: "warp",
  description: "Teleport to place",
  usage: "warp <place_name>",
  group: "Warp"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the place name!")
    let name = args[0]
    let warp
    WarpDB.forEach((key, value) => {
      if (key.startsWith(`Warp-`) && key.substring(`Warp-`.length).toLowerCase() == name.toLowerCase()) {
        warp = value
        return;
      }
    })
    if (warp != undefined) {
      player.tell("§eTeleporting...")
      await tpPlayer(player, `${warp.x} ${warp.y} ${warp.z}`, warp.dimension)
      player.tell("§aSuccessfully Teleport.")
    } else {
      player.tell("§4There's no place with that name!")
    }
  }
)

// LISTWARP
CommandBuild.register({
  name: "listwarp",
  description: "Get all list of place has been set",
  usage: "listwarp",
  group: "Warp"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let message = ""
    WarpDB.forEach((key, value) => {
      let placeName = key.substring(`Warp-`.length)
      message += `\n§e  -§a ${placeName}`;
    })
    if (message != "") {
      player.tell(`§eWarp List : ${message}`)
    }
    else {
      player.tell("§4There's no place or warp!")
    }
  }
)

// -- [TPA] -- \\

// CHECK TPA
/**
 * @param {Player} player 
 */
function noTPA(player) {
  let tag = player.getTags().find(t => t.startsWith("tpasetting"))
  let set = ""
  if (tag == undefined) {
    return false
  }
  set = tag.substring("tpasetting:".length)
  if (set == "on") return false
  if (set == "off") return true
}

// TPA
CommandBuild.register({
  name: "tpa",
  description: "Send request to other player for teleport to him",
  usage: "tpa <player_name>",
  group: "TPA"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    if (noTPA(player)) return player.tell("§4Your's tpa setting is off!")
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      if (targetPlayer == player) return player.tell("§4You can't send tpa request to yourself!")
      if (noTPA(targetPlayer)) return player.tell(`§4${targetPlayer.nameTag}'s tpa setting is off!`)
      if (tpaCooldown.find(p => p.name == player.nameTag) != undefined) return player.tell(`§4You just use tpa command! In cooldown for §e${tpaCooldown.find(p => p.name == player.nameTag).cooldown}s`)
      const tpaForm = new MessageFormData()
        .title("TPA Request")
        .body(`${player.nameTag} sent a request to teleport to you!`)
        .button1("§l§aACCEPT")
        .button2("§l§4DECLINE")

      targetPlayer.tell(`§e${player.nameTag} sent a request to teleport to you!`)
      player.tell(`§aRequest sent, wait for ${targetPlayer.nameTag} to accept!`)
      tpaCooldown.push({
        name: player.nameTag,
        cooldown: 60
      })
      const res = await ForceOpen(targetPlayer, tpaForm)
      if (res.canceled || res.selection == 0) {
        player.tell(`§4${targetPlayer.nameTag} has declined your request!`)
      } else {
        player.tell(`§a${targetPlayer.nameTag} has accepted your request! Teleporting...`)
        await tpPlayer(player, targetPlayer)
        player.tell("§aSuccessfully Teleport.")
      }
    } else {
      player.tell("§4There's no player with that name!")
    }
  }
)

// TPAHERE
CommandBuild.register({
  name: "tpahere",
  description: "Send request to other player for teleport to you",
  usage: "tpahere <player_name>",
  group: "TPA"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    if (noTPA(player)) return player.tell("§4Your's tpa setting is off!")
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      if (targetPlayer == player) return player.tell("§4You can't send tpa request to yourself!")
      if (noTPA(targetPlayer)) return player.tell(`§4${targetPlayer.nameTag}'s tpa setting is off!`)
      if (tpaCooldown.find(p => p.name == player.nameTag) != undefined) return player.tell(`§4You just use tpa command! In cooldown for §e${tpaCooldown.find(p => p.name == player.nameTag).cooldown}s`)
      const tpaForm = new MessageFormData()
        .title("TPAHERE Request")
        .body(`${player.nameTag} sent request to you for teleport to him!`)
        .button1("§l§aACCEPT")
        .button2("§l§4DECLINE")

      targetPlayer.tell(`§e${player.nameTag} sent request to you for teleport to him!`)
      player.tell(`§aRequest sent, wait for ${targetPlayer.nameTag} to accept!`)
      tpaCooldown.push({
        name: player.nameTag,
        cooldown: 60
      })
      const res = await ForceOpen(targetPlayer, tpaForm)
      if (res.canceled || res.selection == 0) {
        player.tell(`§4${targetPlayer.nameTag} has declined your request!`)
      } else {
        player.tell(`§a${targetPlayer.nameTag} has accepted your request!`)
        targetPlayer.tell("§aTeleporting...")
        await tpPlayer(targetPlayer, player)
        targetPlayer.tell("§aSuccessfully Teleport.")
      }
    } else {
      player.tell("§4There's no player with that name!")
    }
  }
)

// TPASETTING
CommandBuild.register({
  name: "tpasetting",
  description: "Set your tpa setting",
  usage: "tpasetting <on/off>",
  group: "TPA"
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) {
      if (noTPA(player)) return player.tell("§eYour's TPA is off!")
      return player.tell("§eYour's TPA is on!")
    }
    if (args[0] != "on" && args[0] != "off") return player.tell("§4You only can input on/off!")
    let prevSet = player.getTags().find(t => t.startsWith("tpasetting"))
    if (prevSet) {
      player.removeTag(prevSet)
    }
    player.addTag("tpasetting:" + args[0])
    player.tell("§aSuccessfully updated your setting to " + args[0])
  }
)

// -- [ADMIN] -- \\

// SETMONEY
CommandBuild.register({
  name: "setmoney",
  description: "Set player's money",
  usage: "setmoney <player_name> <amount>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    if (!args[1]) return player.tell("§4You must input the amount!")
    let amount = Number(args[1])
    if (!Number.isInteger(amount)) return player.tell("§4You must input number!")
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      await updateMoney(targetPlayer, amount)
      player.tell(`§aSuccessfully set ${targetPlayer.nameTag}'s money to §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    } else {
      player.tell("§4There's no player with that name!")
    }
  }
)

// SETWARP
CommandBuild.register({
  name: "setwarp",
  description: "Set place that anyone can teleport to there",
  usage: "setwarp <place_name>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the place name!")
    let name = args[0]
    let warp
    WarpDB.forEach((key, value) => {
      if (key.startsWith(`Warp-`) && key.substring(`Warp-`.length).toLowerCase() == name.toLowerCase()) {
        warp = value
        return;
      }
    })
    if (warp != undefined) return player.tell("§4There's already have place with that name!")
    const placeObject = {
      x: Math.floor(player.location.x),
      y: Math.floor(player.location.y),
      z: Math.floor(player.location.z),
      dimension: getCurrentDimension(player)
    }
    await WarpDB.set(`Warp-${name}`, placeObject)
    player.tell(`§aSuccessfully add warp place with name ${name}!`)
  }
)

// DELWARP
CommandBuild.register({
  name: "delwarp",
  description: "Delete warp place",
  usage: "delwarp <place_name>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the place name!")
    let name = args[0]
    let warp
    WarpDB.forEach((key, value) => {
      if (key.startsWith(`Warp-`) && key.substring(`Warp-`.length).toLowerCase() == name.toLowerCase()) {
        warp = key
        return;
      }
    })
    if (warp == undefined) return player.tell("§4There's no place with that name!")
    await WarpDB.delete(warp)
    player.tell(`§aSuccessfully remove warp place with name ${args[0]}!`)
  }
)

// REFRESHSTOCK
CommandBuild.register({
  name: "refreshstock",
  description: "Refresh Shop Stock",
  usage: "refreshstock",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    await refreshStock()
    player.tell("§aSuccessfully refreshed stock!")
  }
)

// BROADCAST
CommandBuild.register({
  name: "broadcast",
  description: "Send broadcast message to all player",
  aliases: ["bc"],
  usage: "broadcast <message>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    await player.runCommandAsync(`tellraw @a {"rawtext": [{"text": "§4§l[Broadcast]§r ${args.slice(0).join(" ")}"}]}`)
    player.tell("§aBroadcast has successfully sent!")
  }
)

// BAN
CommandBuild.register({
  name: "ban",
  description: "Ban player",
  usage: "ban <player_name>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    if (!args[1]) return player.tell("§4You must input the reason!")
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      await BanDB.set(targetPlayer.nameTag, {
        reason: args.slice(1).join(" "),
        by: player.nameTag
      })
      runCommand(`kick "${targetPlayer.nameTag}" \n§4§lYou're have been banned\nReason : §e${args.slice(1).join(" ")} \n§4By : §e${player.nameTag}`)
      player.tell(`§aSuccessfully banned ${targetPlayer.nameTag}`)
    } else {
      player.tell("§4There's no player with that name!")
    }
  }
)

// UNBAN
CommandBuild.register({
  name: "unban",
  description: "Unban player",
  usage: "unban <player_name>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input the player name!")
    let targetPlayer
    BanDB.forEach((key, value) => {
      if (key.toLowerCase() == args[0].toLowerCase().replaceAll("_", " ")) {
        targetPlayer = key
      }
    })
    if (targetPlayer != undefined) {
      await BanDB.delete(targetPlayer)
      player.tell(`§aSuccessfully unbanned ${targetPlayer}`)
    } else {
      player.tell("§4There's no banned player with that name!")
    }
  }
)

// Setting
CommandBuild.register({
  name: "setting",
  description: "Set configuration for addon",
  usage: "setting <name> <value>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    let settingList = {
      "moneySystem": SettingType.TRUEFALSE,
      "homeSystem": SettingType.TRUEFALSE,
      "tpaSystem": SettingType.TRUEFALSE,
      "shopSystem": SettingType.TRUEFALSE,
      "refreshStock": SettingType.TRUEFALSE,
      "starterMoney": SettingType.CUSTOMNUM,
      "earnMoneyfromMobs": SettingType.TRUEFALSE,
      "earnMoneyfromPlayers": SettingType.TRUEFALSE
    }
    if (!args[0]) return player.tell("§4You must input the setting name!")
    if (args[0] == "list") {
      let message = `§aSetting List :`
      Object.keys(settingList).forEach(name => {
        message += `\n §e - §a${name} §e| `
        let valueType = settingList[name]
        if (valueType == SettingType.TRUEFALSE) {
          message += `${name} <value: true/false>`
        } else if (valueType == SettingType.CUSTOMNUM) {
          message += `${name} <value: number>`
        } else if (valueType == SettingType.CUSTOMSTR) {
          message += `${name} <value: string>`
        } else {
          message += `§4[Error] No value type!`
        }
      })
      return player.tell(message)
    }
    let selectedName = Object.keys(settingList).find(s => s.toLowerCase() == args[0].toLowerCase())
    if (selectedName == undefined) return player.tell("§4There's no setting with that name! Use " + prefix + "list to get setting list.")
    let selectedType = settingList[selectedName]
    if (!args[1]) return player.tell("§4You must input the value")
    let value = args[1].toLowerCase()
    switch (selectedType) {
      case SettingType.TRUEFALSE:
        if (value != "true" && value != "false") return player.tell("§4You only can input true/false!")
        await ServerSetDB.set(selectedName, value)
        if (selectedName == "refreshStock")
          await refreshStock()
        player.tell("§aSuccessfully set!")
        break;
      case SettingType.CUSTOMSTR:
        await ServerSetDB.set(selectedName, value)
        player.tell("§aSuccessfully set!")
        break;
      case SettingType.CUSTOMNUM:
        value = Number(value)
        if (!Number.isInteger(value)) return player.tell("§4You must input number!")
        await ServerSetDB.set(selectedName, value)
        player.tell("§aSuccessfully set!")
        break;
      default:
        player.tell("§4[ERROR] The setting has no value type!")
    }
  }
)

/**
 * @param {Player} player
 * @returns {String}
 */
async function selectGroup(player) {
  let groups = Object.keys(getSetting("itemShop"))
  let form = new ModalFormData()
    .title("Select Group")
    .dropdown("Select Group :", groups)

  let res = await ForceOpen(player, form)
  if (!res.canceled) {
    let group = groups[res.formValues[0]]
    return group
  }
  return undefined;
}

CommandBuild.register({
  name: "shopsetting",
  description: "Edit item/group in Shop",
  usage: "shopsetting <item/group> <add/remove/edit>",
  group: "Admin",
  admin: true
},
  /**
   * @param {BeforeChatEvent} data 
   * @param {Player} player 
   * @param {String} args 
   */
  async (data, player, args) => {
    if (!args[0]) return player.tell("§4You must input group/item!")
    if (args[0].toLowerCase() == "group") {
      if (!args[1]) return player.tell("§4You must input the operations! (add/remove/edit)")
      if (args[1].toLowerCase() == "add") {
        player.tell("§eClose Chat to Show UI!")
        return addGroup(player)
      } else if (args[1].toLowerCase() == "remove") {
        player.tell("§eClose Chat to Show UI!")
        let group = await selectGroup(player)
        if (group == undefined) return;
        return removeGroup(player, group)
      }
    } else if (args[0].toLowerCase() == "item") {
      if (!args[1]) return player.tell("§4You must input the operations! (add/remove/edit)")
      if (args[1].toLowerCase() == "add") {
        let selectedItem = player.getComponent("inventory").container.getItem(player.selectedSlot)
        if (selectedItem == undefined) return player.tell("§4You must hold the item that you want to add!")
        player.tell("§eClose Chat to Show UI!")
        let group = await selectGroup(player)
        if (group == undefined) return;
        return addItem(player, group, selectedItem)
      } else if (args[1].toLowerCase() == "remove") {
        player.tell("§eClose Chat to Show UI!")
        let group = await selectGroup(player)
        if (group == undefined) return;
        return removeItem(player, group)
      } else if (args[1].toLowerCase() == "edit") {
        player.tell("§eClose Chat to Show UI!")
        let group = await selectGroup(player)
        if (group == undefined) return;
        return editItem(player, group)
      }
    } else return player.tell("§4You must input group/item!")
  }
)

/**
 * @param {Player} player 
 * @param {String} group
 * @param {ItemStack} item
 */
async function addItem(player, group, item) {
  let form = new ModalFormData()
    .title("Adding Item to Shop")
    .textField("Item Id :", "Example : rawiron")
    .textField("Item Display Name :", "Example : Raw Iron")
    .textField("Item Price :", "Example : 1000")

  let res = await ForceOpen(player, form)
  if (!res.canceled) {
    let itemShop = getSetting("itemShop")
    let [id, name, price] = res.formValues
    if (id == "" || name == "" || price == "") return player.tell("§4You must complete the form!")
    if (itemShop[group].find(i => i.id.toLowerCase() == id.toLowerCase()) != undefined) return player.tell("§4This id already exist!")
    price = Number(price)
    if (!Number.isInteger(price)) return player.tell("§4You must input number!")
    itemShop[group].push({
      id: id,
      itemid: item.typeId,
      data: item.data,
      displayname: name,
      price: price
    })
    await settingShop(itemShop)
    player.tell("§aSuccessfully add item to shop!")
  }
}

/**
 * @param {Player} player 
 * @param {String} group 
 */
async function removeItem(player, group) {
  let itemId = []
  for (const item of getSetting("itemShop")[group]) {
    itemId.push(item.id)
  }
  let form = new ModalFormData()
    .title("Removing Item in Shop")
    .dropdown("Select Item ID :", itemId)

  let res = await ForceOpen(player, form)
  if (!res.canceled) {
    let itemShop = getSetting("itemShop")
    let itemStock = ShopDB.get("currentStock")
    let ID = itemId[res.formValues[0]]
    let item = itemShop[group].findIndex(s => s.id.toLowerCase() == ID.toLowerCase())
    itemShop[group].splice(item, 1)
    let i = itemStock[group].findIndex(s => s.toLowerCase() == ID.toLowerCase())
    itemStock[group].splice(i, 1)

    await settingShop(itemShop)
    await ShopDB.set("currentStock", itemStock)
    player.tell(`§aSuccessfully remove item with id : §e${ID}§a in Shop!`)
  }
}

/**
 * @param {Player} player 
 * @param {String} group 
 */
async function editItem(player, group) {
  let itemId = []
  for (const item of getSetting("itemShop")[group]) {
    itemId.push(item.id)
  }
  let type = ["DisplayName", "Price"]
  let form = new ModalFormData()
    .title("Editing Item in Shop")
    .dropdown("Select Item ID :", itemId)
    .dropdown("Select what do you want to edit :", type)
    .textField("New Value :", "Input Here")

  let res = await ForceOpen(player, form)
  if (!res.canceled) {
    let [id, edit, value] = res.formValues
    if (value == "") return player.tell("§4You must complete the form!")
    id = itemId[id]
    let itemShop = getSetting("itemShop")
    let item = itemShop[group].findIndex(s => s.id.toLowerCase() == id.toLowerCase())
    if (edit == 1 && !Number.isInteger(Number(value))) return player.tell("§4You must input number!")
    type = type[edit]
    itemShop[group][item][type.toLowerCase()] = value

    await settingShop(itemShop)
    player.tell(`§aSuccessfully edit item §e${type}§a to §e${value}`)
  }
}

/**
 * @param {Player} player 
 */
async function addGroup(player) {
  let form = new ModalFormData()
    .title("Add Group")
    .textField("Group Name :", "Example : Blocks")

  let res = await ForceOpen(player, form)
  if (!res.canceled) {
    let itemShop = getSetting("itemShop")
    let [name] = res.formValues
    if (name == "") return player.tell("§4You must complete the form!")
    if (Object.keys(itemShop).find(k => k.toLowerCase() == name) != undefined) return player.tell("§4This group name already exist!")
    itemShop[name] = []

    await settingShop(itemShop)
    player.tell(`§aSuccessfully add group with name §e${name}`)
  }
}

/**
 * @param {Player} player 
 * @param {String} group
 */
async function removeGroup(player, group) {
  let itemShop = getSetting("itemShop")
  let itemStock = ShopDB.get("currentStock")
  group = Object.keys(itemShop).find(k => k.toLowerCase() == group.toLowerCase())
  if (group == undefined) return player.tell("§4There's no group with that name!")
  delete itemShop[group]
  delete itemStock[group]

  await settingShop(itemShop)
  await ShopDB.set("currentStock", itemStock)
  player.tell(`§aSuccessfully remove group with name §e${group}`)
}